!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Module      : omprep_mod
! Authors     : Tim Harries (MSSL)
!               Jon Rainger (MSSL)
!               Phil Williams (MSSL)
!               Andrew King (MSSL)
!
! Description : This tasks interacts directly with the ODF files, cloning the input and 
!               modifying it for subsequent processing  by the pipeline tasks. 
!
! Comments    : SCIWIN convention not stable yet.
!               FAST Mode SAMPTIME and BFAST_ID keywords could be used in place of BINAX1, BINAX2.
!               Origin of RA_PNT, DEC_PNT and ROLL_PNT unknown. Possibly could be taken from SUM file.
!               Value for keyword ORIGIN unknown.
!
! $Id         : module.f90,v 1.4 1997/12/18 16:01:10 gvacanti Exp $
!
! History : Created  : January 98 - TJH
!           Modified : 27 April 98 - Removed datestamp.
!                      12 March 99 - Spring clean of code.
!                      20 April 99 - Change parameter names to conform to new convention.
!                      18 May - Remove dependancies on filename.
!                             - Remove odfdir parameter and call to open OAL.
!                      21 July - Writes sample time and bfast id to event list.
!                      06 Jan - Additional RAW extension added.
!                      17.05.2000 - Removed hardwiring of dimension of 10 for maximum number of 
!                      guide stars although this was correct. WE saw some data with a maximum of 16.
!                      The code now works out the maximum dimension size from the data and uses this
!                      Should work in all cases! 
!                      23.05.2000 AMK - Implemented calls to OAL_getodInfo and OAL_getAttitude
!                      to get telescope attitude. Remomoved hardwiring RA,DEC and PA and replaced
!                      with values obtaind from the above calls 
!                      21.06.2000 AK - Implemented the call to OAL_exposureInfo in order that the
!                      start time, end time and exposure length could be determined.
!                      The call to getExposure has been removed.
!                      20.07.2000 AMK - Added a command line parameter slewset which indicates whether
!                      we are processing slew data or true imaging data. The logical flag slewFlag is set
!                      accordingly. There are two switches in the code that depend upon this slewFlag.
!                      The first switch just sets binBPE to TRUE if slewFlag is TRUE otherwise the code works
!                      out as before how to set binBPE.!                      The second switch will skip the attitude code if slewFlag is TRUE.
!                      21/07/2000 AMK - Added additional code to deal with the case of a missing tracking history file.
!                      OMICHAIN has been modified so that when this situation arises the filename dummyTHX.FIT is passed
!                      in. The code now tests for this situation and set the flag dummyTHXFlag to TRUE. The 
!                      ZERODRIFT option is then implemented.
!                      This device enables us to get around this problem and requiring the user to have a dummy file
!                      present.
!                      21/07/2000 AMK - Added the above creation of a dummy tmp_tracking for the slew optin.
!                      24/07/2000 AMK - Added the subroutine convertExposureStringToInteger. This converts the
!                      string holding the exposure number to an integer. This is in turn passed to getstartendexp.  
!                      24/08/2000 AMK - Added the keyword BLANK to the output image file. This is required later
!                      in the chain by OMFLATFIELD in order to set the quality array.
!                      22/09/2000 AMK - implemented checking of the raw image in order to detect negative pixel counts.
!                      These counts are reset to zero and a warning issued. This is short term fix.
!                      02/10/2000 AMK changed the call t getStartEndExp which now returns the exposure filter.
!                      The code that obtains he filter from the PEH file is now redundant.
!                      18/10/2000 - Filter string initialised to " "
!                      1.11.2000 - The filter string is now checked to ensure it is a valid filter- if not then the
!                      program aborts.
!                      14.11.2000 - DETNAM set to REDUNDANT instead of PRIME if chain=1. 
!                      06.03.2001 - routine getexposurestartrow revised
!                      07.03.2001 - subroutine CheckImageSizeOkay added to ensure that the x and y image dimensions
!                      agree with those computed from the fits file attributes BINBPE, WINDOWDX, WINDOWDY, BINAX1 
!                      and BINAX2
!-
!*************************************************************************************************************************
module omprep_mod

  use param
  use oal
  use caloaldefs
  use dal

  use sasdal
  use errorHandling
!use sla
  use getstartendexp_mod

  !
  !-----------------------------------------------------------------------
  !
  use types
  !
  !----------------------------------------------------------------------
  !
  implicit none

  private

  public omprep
 
contains

  !---------------------------------------------------------------------
  subroutine omprep
    !---------------------------------------------------------------------
    type(DataSetT) :: inDataSetH                                      ! Input dataset handle.
    type(DataSetT) :: pehDataSetH                                     ! Periodic Housekeeping dataset handle.
    type(DataSetT) :: wdxDataSetH                                     ! Priority Window Data dataset handle.
    type(DataSetT) :: outDataSetH                                     ! Output dataset handle.
    type(ArrayT) :: inArrayH                                          ! Input array handle.
    type(ArrayT) :: outArrayH                                         ! Output array handle.
    type(TableT) :: inTableH                                          ! Input table handle.
    type(TableT) :: pehTableH                                         ! Periodic Housekeeping table dataset handle.
    type(TableT) :: wdxTableH                                         ! Priority Window Data table dataset handle.
    type(TableT) :: outTableH                                         ! Output table handle.
    type(TableT) :: modesTableH                                       ! Modes table dataset handle.
    type(ColumnT), dimension(9) :: inColumnH                          ! Input column handle.
    type(ColumnT), dimension(2) :: wdxColumnH                         ! Priority Window Data column handle.
    type(ColumnT), dimension(9) :: outColumnH                         ! Column handle.
    type(ColumnT), dimension(5) :: modesColumnH                       ! Modes column handle.
    type(ColumnT) :: H5215Col
    type(ProposalInfoType) :: propInfo                                ! Proposal info data.

    character(len=3) :: fileType                                      ! Input file file type.
    character(len=10) :: oswId                                        ! Observation Science Window identifier.
    character(len=20) :: object                                       ! Name of observed object.
    character(len=80) :: observer                                     ! Observer's name.
    character(len=255) :: set                                         ! OM ODF Image / Event List / Tracking History Data Auxiliary filename.
    character(len=255) :: pehSet                                      ! OM ODF Periodic Housekeeping filename.
    character(len=255) :: nphSet                                      ! OM ODF Non-Periodic Housekeeping filename.
    character(len=255) :: wdxSet                                      ! OM ODF Priority Window Data Auxiliary filename.
    character(len=255) :: outSet                                      ! OM image / event list intermdiate output filename.
    character(len=255) :: telescope                                   ! Keywords.
    character(len=255) :: instrument                                  ! Keywords.
    character(len=255) :: dataType                                    ! Keywords.
    character(len=255) :: obsId                                       ! Keywords.
    character(len=255) :: expId                                       ! Keywords.
    character(len=255) dateObs                                        ! Keywords.
    character(len=255) dateEnd                                        ! Keywords.
    character(len=255) detSec, detsize                                ! Keywords.   

    logical :: binBPE                                                 ! BPE binning flag.

    integer(kind=int16), dimension(:), pointer :: ftFineP             ! Fine time column pointer.
    integer(kind=int16), dimension(:), pointer :: rawXP               ! Raw x column pointer.
    integer(kind=int16), dimension(:), pointer :: rawYP               ! Raw y column pointer.
    integer(kind=int16), dimension(:), pointer :: int16ColumnP        ! Column pointer.
    integer(kind=int16) :: mode                                       ! Keywords.

    integer(kind=int32), dimension(:), pointer :: frameP              ! Frame column pointer.
    integer(kind=int32), dimension(:), pointer :: ftCoarseP           ! Coarse time column pointer.
    integer(kind=int32), dimension(:), pointer :: nggsP               ! No. good guide stars column pointer.
    integer(kind=int32), dimension(:), pointer :: dxP                 ! Drift in X column pointer.
    integer(kind=int32), dimension(:), pointer :: dyP                 ! Drift in Y column pointer.
    integer(kind=int32), dimension(:), pointer :: rollP               ! Roll column pointer.
    integer(kind=int32), dimension(:), pointer :: qualityP            ! Quality column pointer.
    integer(kind=int32), dimension(:,:), pointer :: gsXP              ! Guide Star X coordinate column pointer.
    integer(kind=int32), dimension(:,:), pointer :: gsYP              ! Guide Star Y coordinate column pointer.
    integer(kind=int32), dimension(:,:), pointer :: gsCtsP            ! Guide Star Counts column pointer.
    integer(kind=int32), dimension(:, :), pointer :: binAx1P          ! Binax1 pointer.
    integer(kind=int32), dimension(:, :), pointer :: binAx2P          ! Binax2 pointer.
    integer(kind=int32), dimension(:), pointer :: modeP               ! Mode pointer.
    integer(kind=int32), dimension(:), pointer :: windowX0P           ! WindowX0 pointer.
    integer(kind=int32), dimension(:), pointer :: windowY0P           ! WindowY0 pointer.
    integer(kind=int32), dimension(:), pointer :: windowDXP           ! WindowDX pointer.
    integer(kind=int32), dimension(:), pointer :: windowDYP           ! WindowDY pointer.
    integer(kind=int32), dimension(:), pointer :: int32ColumnP        ! Column pointer.
    integer(kind=int32), dimension(:, :), pointer :: int32VectColumnP ! Vector Column pointer.
    integer(kind=int32) :: chain                                      ! Keywords.
    integer(kind=int32) :: tmExpId                                    ! Keywords.
    integer(kind=int32) :: sciWin                                     ! Keywords.
    integer(kind=int32) :: frmTime                                    ! Keywords.
    integer(kind=int32) :: windowX0                                   ! Keywords.
    integer(kind=int32) :: windowY0                                   ! Keywords.
    integer(kind=int32) :: windowDX                                   ! Keywords.
    integer(kind=int32) :: windowDY                                   ! Keywords.
    integer(kind=int32) :: binX, binY                                 ! Binning
    integer(kind=int32), dimension(:, :), pointer :: int32ImageP      ! Integer array pointer.
    integer(kind=int32), dimension(2) :: imageDim                     ! Image dimension.
    integer(kind=int32) :: nEvents                                    ! Number of events.
    integer(kind=int32) :: nFrames                                    ! Number of tracking frames.
    integer(kind=int32) :: binAx1                                     ! DPU x-axis binning factors.
    integer(kind=int32) :: binAx2                                     ! DPU y-axis binning factors.
    integer(kind=int32) :: xAxisLength                                ! Length of image x-axis.
    integer(kind=int32) :: yAxisLength                                ! Length of image y-axis.

    integer(kind=int32), dimension(1) :: columnDim                    ! Column dimension.
    integer :: imageDimensions(2)
    real(kind=single), dimension(:, :), pointer :: real32ImageP       ! Real array pointer.
    real(kind=single) :: exposure                                     ! Exposure length.
    

    !///////////////////////////////////////////////////////////////////////////////////////////////////////////
    !
    ! AMK - 23.05.2000
    !
    ! Additional declarations required for attitude information
    !
    !//////////////////////////////////////////////////////////////////////////////////////////////////////////

    type(SpacecraftAttitudeType)                    :: attitude               ! Attitude information

    character(len=3)                                :: exposureNumber         ! Strying to hold exposure number.
    character(len=12)                               :: string
    
    character(len=32)                               :: filterString=" " ! Exposure filter string.
                                                                             
    integer                                         :: exposureStartRow
    integer                                         :: expNumber              ! Exposure Number.
    !integer                                         :: istat                  ! Status flag for memory allocation
    integer(kind=single)                            :: length1                ! String length.
    integer(kind=single)                            :: length2                ! String length.
    integer(kind=int32)                             :: blank

    real, parameter                                 :: radian = 57.29577951E0 ! Converts radians to degrees
    real (kind=double)                              :: matrix(3, 3)
    real(double)                                    :: attrarad = 0.0d0               ! RA of Attitude in radians
    real(double)                                    :: attdecrad = 0.0d0              ! DEC of Attitude in radians
    real(double)                                    :: attPArad = 0.0d0               ! PA of Attitude in radians
    real(kind=double)                               :: OBTstart = 0.d0                ! Start time.
    real(kind=double)                               :: OBTend =0.d0                   ! End time.

    logical                                         :: slewFlag               ! Indicates if we are processing
    ! data or imaging data.
    logical                                         :: dummyTHXFlag           ! Flag to indicate the absence of
    ! a tracking history file.

    integer :: i, j
                                

    !//////////////////////////////////////////////////////////////////////////////////////////////////////////
    !
    ! Read in parameters.

    set = stringParameter("set")
    pehSet = stringParameter("pehset")
    nphSet = stringParameter("nphset")
    wdxSet = stringParameter("wdxset")
    outSet = stringParameter("outset")
    slewFlag = booleanParameter("slewset")

    !-----------------------------------------------------------------------------------

    ! 
    ! AMK - 21/07/2000
    !
    ! Check to see if we have a dummy tracking file name. Set dummyTHXFlag correctly.
    !
    if(set .eq. "DUMMYTHX.FIT") then
       dummyTHXFlag = .TRUE.
    else
       dummyTHXFlag = .FALSE.
    end if

    ! Read in ODF Image / Event List / Tracking History Data Auxiliary file.
    !
    ! AMK - 21/07/2000
    !
    ! Added!
    THXFLAG: if(.not. dummyTHXFlag) then
       inDataSetH = dataSet(set, read)

       !**************************************
       !  Start of "OMIMI1" if block.
       !**************************************

       OMIMI1: if (hasBlock(inDataSetH, "OMIMI1")) then   
          ! OM Imaging Mode Data Image file.
          fileType = "IMI"
          ! Get a handle on the image array extension.
          inArrayH = array(inDataSetH, "OMIMI1", read)

 
          ! Read in the image array extension keywords.
          telescope = stringAttribute(inArrayH, "TELESCOP")
          instrument = stringAttribute(inArrayH, "INSTRUME")
          chain = int32Attribute(inArrayH, "CHAIN")
          dataType = stringAttribute(inArrayH, "DATATYPE")
          obsId = stringAttribute(inArrayH, "OBS_ID")
          expId = stringAttribute(inArrayH, "EXP_ID")
          tmExpId = int32Attribute(inArrayH, "TMEXP_ID")
          sciWin = int32Attribute(inArrayH, "SCIWIN")
          frmTime = int32Attribute(inArrayH, "FRMTIME")
          windowX0 = int32Attribute(inArrayH, "WINDOWX0")
          windowY0 = int32Attribute(inArrayH, "WINDOWY0")
          windowDx = int32Attribute(inArrayH, "WINDOWDX")
          windowDy = int32Attribute(inArrayH, "WINDOWDY")

          binAx1 = int32Attribute(inArrayH, "BINAX1")
          binAx2 = int32Attribute(inArrayH, "BINAX2")

          !**********************************************
          ! Get the actual dimensions of the image stored 
          ! int the file
          !*********************************************
          imageDimensions = dimensions(inArrayH)
          xAxisLength = imageDimensions(1)
          yaxisLength = imageDimensions(2) 

          dateObs = stringAttribute(inArrayH, "DATE-OBS")
          dateEnd = stringAttribute(inArrayH, "DATE-END")

          !
          !----------------------------------------------------------
          !
          ! AMK - 23/08/2000
          !
          ! We need to read the BLANK keyword and pass this into the output image. This is needed by OMFLATFLELD
          ! for later preocessing in order to deal with flatfield elements that zero
          !
          blank = int32Attribute(inArrayH, "BLANK")

          !---------------------------------------------------------
          !
          ! Get a pointer to the image.
          int32ImageP => int32Array2Data(inArrayH)

          ! Get the dimensions of the image.
          imageDim = dimensions(inArrayH)

          !
          !------------------------------------------------------------
          !
          ! AMK - 22/09/2000
          ! 
          ! We need to check the image for negative counts. If this occurs
          ! we set the pixel to value BLANK as the system already copes 
          ! with this. A system warning is issued.
          ! This must be taken as a temporary solution.
          !
          !------------------------------------------------------------
          !
          do i = 0 , imageDim(1) - 1
             do j = 0 , imageDim(2) - 1
                if(int32ImageP(i, j) .lt. 0) then
                   int32ImageP(i, j) = 0.0
                   write (errstr, *) "Negative Pixel value at ",i,j," Resetting to zero"         
                   call warning("NEGATIVEPIXELVALUE")
                end if
             end do
          end do

          !
          !----------------------------------------------------------
          !

          !
          !----------------------------------------------------------
          !

          !-------------------------------------------------------------------------------------------------
       else if (hasBlock(inDataSetH, "OMFAE1")) then   ! "OMFAE1" present.
          ! OM Fast Mode Event List file.
          fileType = "FAE"

          ! Get a handle on the event list binary table extension.
          inTableH = table(inDataSetH, "OMFAE1")

          ! Read in the event list binary table extension keywords.
          telescope = stringAttribute(inTableH, "TELESCOP")
          instrument = stringAttribute(inTableH, "INSTRUME")
          chain = int32Attribute(inTableH, "CHAIN")
          dataType = stringAttribute(inTableH, "DATATYPE")
          obsId = stringAttribute(inTableH, "OBS_ID")
          expId = stringAttribute(inTableH, "EXP_ID")
          tmExpId = int32Attribute(inTableH, "TMEXP_ID")
          sciWin = int32Attribute(inTableH, "SCIWIN")
          frmTime = int32Attribute(inTableH, "FRMTIME")
          windowX0 = int32Attribute(inTableH, "WINDOWX0")
          windowY0 = int32Attribute(inTableH, "WINDOWY0")
          windowDX = int32Attribute(inTableH, "WINDOWDX")
          windowDY = int32Attribute(inTableH, "WINDOWDY")
          dateObs = stringAttribute(inTableH, "DATE-OBS")
          dateEnd = stringAttribute(inTableH, "DATE-END")

          ! Get handles and pointers on the event list binary table extension columns.
          inColumnH(1) = column(inTableH, "FRAME", read)
          frameP => int32Data(inColumnH(1))
          inColumnH(2) = column(inTableH, "FTCOARSE", read)
          ftCoarseP =>  int32Data(inColumnH(2))
          inColumnH(3) = column(inTableH, "FTFINE", read)
          ftFineP => int16Data(inColumnH(3))
          inColumnH(4) = column(inTableH, "RAWX", read)
          rawXP => int16Data(inColumnH(4))
          inColumnH(5) = column(inTableH, "RAWY", read)
          rawYP => int16Data(inColumnH(5))

          ! Get the number of rows in the event list binary table extension.
          nEvents = numberOfRows(inTableH)

       else if (hasBlock(inDataSetH, "OMTHX1")) then  ! "OMTHX1" present.
          ! OM Tracking History Data Auxiliary file.
          fileType = "THX"
          ! Get a handle on the event list binary table extension.
          inTableH = table(inDataSetH, "OMTHX1")

          ! Read in the tracking history binary table extension keywords.
          telescope = stringAttribute(inTableH, "TELESCOP")
          instrument = stringAttribute(inTableH, "INSTRUME")
          chain = int32Attribute(inTableH, "CHAIN")
          dataType = stringAttribute(inTableH, "DATATYPE")
          obsId = stringAttribute(inTableH, "OBS_ID")
          expId = stringAttribute(inTableH, "EXP_ID")
          tmExpId = int32Attribute(inTableH, "TMEXP_ID")
          frmTime = int32Attribute(inTableH, "FRMTIME")
          dateObs = stringAttribute(inTableH, "DATE-OBS")
          dateEnd = stringAttribute(inTableH, "DATE-END")

          ! Get handles and pointers on the tracking history binary table extension columns.
          inColumnH(1) = column(inTableH, "FRAME", read)
          frameP => int32Data(inColumnH(1))
          inColumnH(2) = column(inTableH, "NGGS", read)
          nggsP => int32Data(inColumnH(2))
          inColumnH(3) = column(inTableH, "DX", read)
          dXP => int32Data(inColumnH(3))
          inColumnH(4) = column(inTableH, "DY", read)
          dYP => int32Data(inColumnH(4))
          inColumnH(5) = column(inTableH, "ROLL", read)
          rollP => int32Data(inColumnH(5))
          inColumnH(6) = column(inTableH, "QUALITY", read)
          qualityP=> int32Data(inColumnH(6))
          inColumnH(7) = column(inTableH, "GSX", read)
          gsXP => int32VectorData(inColumnH(7))
          inColumnH(8) = column(inTableH, "GSY", read)
          gsYP => int32VectorData(inColumnH(8))
          inColumnH(9) = column(inTableH, "GSCTS", read)
          gsCtsP => int32VectorData(inColumnH(9))

          ! Get the number of frames in the tracking history binary table extension.
          nFrames = numberOfRows(inTableH)

       else if (hasBlock(inDataSetH, "ZERODRIFT")) then   ! "ZERODRIFT" present.
          write (errstr, *) "Missing OM Tracking History Data Auxiliary File. Using ZERODRIFT option"      
          call warning("ZERODRIFT")
       else
          ! Incompatible file.

          write (errstr, *) "Could not find any compatibile blocks in set ", set
          call fatal("badFileType")
          !********************************************************
          !  End of OMIMI1 if block.
          !********************************************************
       end if OMIMI1
       !
       ! AMK - 21/07/2000
       !
       ! Added to deal with the lack of a Tracking History File
       !
    else ! This is for missing THX files
       write (errstr, *) "Missing OM Tracking History Data Auxiliary File. Using ZERODRIFT option"      
       call warning("ZERODRIFT")
    end if THXFLAG

    !
    ! AMK - 21/07/2000
    !
    ! Added
    !
    THXFLAG1: if(.not. dummyTHXFlag) then
       !***********************************************************
       ! Start of ZERODRIFT if block - new bit added by JFR on 24.3.2000.
       !***********************************************************
       ZERODRIFT:if (hasBlock(inDataSetH, "ZERODRIFT")) then

          !****  Replace with clone when available JFR *****

          outDataSetH = dataSet(outSet, create)

          outTableH = addTable(outDataSetH, "ZERODRIFT", 1)

          call copyAllAttributes(attributable(inTableH), attributable(outTableH))

          outColumnH(1) = addColumn(outTableH, "DRIFT", integer32)
          int32ColumnP => int32Data(outColumnH(1))
          int32ColumnP = 0.0

          call release(outColumnH(1))
          call release(outTableH)

       else
          ! Read in OM ODF Periodic Housekeeping file.
          pehDataSetH = dataSet(pehSet, read)

          !******************************************
          ! Start of OMPEH1 if block.
          !******************************************
          OMPEH1: if (hasBlock(pehDataSetH, "OMPEH1")) then

             !
             ! AMK - 20-07-2000
             !
             ! Determine if we are deaing with slew data or imaging.
             ! If we processing slew data then just set the binBPE to TRUE othyerwise preocess as below
             !
             if(slewFlag) then
                binBPE = .true.
             end if
             !
             ! AMK - 19-07-2000
             !
             ! We must find the accurate start of the exposure for the NPH HK file first
             !
             SLEWFLAG1: if(.not. slewFlag) then
                !
                ! Get lengths of the strings holding the observation ID and the exposure Id.
                !
                length1 = len_trim(expId)
                length2 = len_trim(obsId)

                !
                ! Set the string for the exposure number
                !
                exposureNumber = expId(length2 + 1 : length1)

                !
                ! Now convert the string to an integer
                !
                !               call convertExposureStringToInteger(exposureNumber ,expNumber)

                read(exposureNumber,'(i3)') expNumber

                !
                ! Now find the accurate start and end times for this exposure.
        
                !              
  

                call getstartendexp(nphSet, expNumber, OBTstart, OBTend, dateObs, dateEnd, filterString)

              
                !*****************************************
                ! Convert the string to upper-case characters
                !*****************************************
                call ConvertToUpperCase(filterString)
 
                !*******************************************************
                ! Check that the filter string is okay (added 31.10.2000)
                !*******************************************************                 
                if (IsBadFilter(filterString)) then
                    call fatal("Invalid filter string", filterString)
                end if

                ! Get a handle on the periodic housekeeping binary table extension.
                pehTableH = table(pehDataSetH, "OMPEH1")

                !*****************************************************************************************
                ! Determine the exposure start time row from the PEH KH file. This is used the resolution.
                !***************************************************************************************** 
               
                call getExposureStartRow(pehTableH, numberOfRows(pehTableH), dateObs ,exposureStartRow) 
         
 
        
               ! Get acquisution mode
                H5215Col = column(pehTableH, "H5215", read)
                string = stringCell(H5215col,exposureStartRow)
                if ((string(1:10) == "LO RES WIN") .or. (stringCell( H5215col, exposureStartRow ) == "LORESFUL")) then

                   binBPE = .true.
                elseif ((string(1:10) == "HI RES WIN") .or. (stringCell( H5215col, exposureStartRow ) == "HIRESFUL")) then  
                   binBPE = .false.  
                else

                   !               write (errstr, *) "Data taken in Engineering Mode ", stringCell( H5215col, midRow ) 
                   write (errstr, *) "Data taken in Engineering Mode ", stringCell( H5215col, exposureStartRow ) 

                   call fatal("BadAcqMode")

                end if

                call release(H5215Col)


                ! Release handles on column and table.
                call release(pehTableH)
                !***************************************************************
                ! Check that the image dimensions match those inferred from the
                ! header information and binning factors
                !***************************************************************
               
                if (fileType .eq. "IMI") call CheckImageSizeOkay(binBPE, windowDX, windowDY, binAx1, &
                     binAx2, xAxisLength, yAxisLength) 
              

             end if SLEWFLAG1

          else ! Incompatible file.
             write (errstr, *) "Could not find any compatibile blocks in set ", pehSet
             call fatal("badFileType")
             !************************************************
             ! End of OMPEH1 block.
             !***********************************************
          end if OMPEH1

          ! Release handles on data.
          call release(pehDataSetH)

          !
          ! AMK - 05/07/2000 Removed time exposure code to subroutine now done above in getStartTime 
          !

          !-----------------------------------------------------------------------------

          ! Set the state of the OAL

  
          !
          ! Find the exposure length - could probably be done earlier.
          !
          exposure = OBTend - OBTstart

          ! Read in OM ODF Priority Window Data Auxiliary file.

          binax1 = 0.0
          binax2 = 0.0

          if (fileType == "IMI" .or. fileType == "FAE" ) then

             wdxDataSetH = dataSet(wdxSet, read)

             if (hasBlock(wdxDataSetH, "OMWDX1")) then
                ! Get a handle on the window data binary table extension.
                wdxTableH = table(wdxDataSetH, "OMWDX1")

                ! binAx1 is found in SWP1 and binAx2 in SWP2.
                ! PSW sciWin numbering system still not settled.
                ! May add sampling time and bfast ID to event list.

                wdxColumnH(1) = column(wdxTableH, "SWP1", read)
                binAx1P => int32VectorData(wdxColumnH(1))
                binAx1 = binAx1P(sciWin, 0)
                wdxColumnH(2) = column(wdxTableH, "SWP2", read)
                binAx2P => int32VectorData(wdxColumnH(2))
                binAx2 = binAx2P(sciWin, 0)

                ! Release handles on data.
                call release(wdxColumnH(1))
                call release(wdxColumnH(2))
                call release(wdxTableH)
                call release(wdxDataSetH)
             else ! Incompatible file.
                !            write (errstr, *) "Could not find any compatibile blocks in set ", wdxSet
                call fatal("badFileType")
             end if
          end if

          ! Get the proposal info. from observation summary file.
          call OAL_proposalInfo(propInfo)

          observer = propInfo%GoName
          object = propInfo%targetName

          ! Write out OM image / event list / tracking history intermdiate output filename.
          outDataSetH = dataSet(outSet, create)


          call setAttribute(outDataSetH, "TELESCOP", trim(telescope), "Telescope (mission) name")
          call setAttribute(outDataSetH, "INSTRUME", trim(instrument), "Instrument name")

          ! Chain tells us which detector is in use.
          if (chain == 1) then
             call setAttribute(outDataSetH, "DETNAM", "REDUNDANT", "PRIME or REDUNDANT")
          else
             call setAttribute(outDataSetH, "DETNAM", "PRIME", "PRIME or REDUNDANT")
          endif

          ! Write DATAMODE keyword to outSet.
          if (trim(dataType) .eq. "IMG.IM") then
             call setAttribute(outDataSetH, "DATAMODE", "IMAGING", "Instrument mode")
          else if (trim(dataType) .eq. "FST.EL") then
             call setAttribute(outDataSetH, "DATAMODE", "FAST", "Instrument mode")
          else if (trim(dataType) .eq. "TRH.EL") then
             call setAttribute(outDataSetH, "DATAMODE", "TRACKING", "Instrument mode")
          else
             call fatal("wrongMode", "OM ODF file not taken in imaging/fast/tracking mode")
             !      call setAttribute(outDataSetH, "DATAMODE", "UNKNOWN", "Instrument mode")
          endif

          ! Write FILTER keyword to outSet.

          !if (iFilter > 10) then
          !   call fatal("badFilt", "Bad filter position in housekeeping")
          !end if

!
!---------------------------------------------------------------------------------------------
!
! AMK - 02/10/2000
!
! Changed below as we now have the filter string
!
          !call setAttribute(outDataSetH, "FILTER", trim(filter(iFilter)), "Filter ID")
          call setAttribute(outDataSetH, "FILTER", trim(filterString), "Filter ID")
!
!---------------------------------------------------------------------------------------------
!
          ! Write keywords to outSet.

          call setAttribute(outDataSetH, "OBS_MODE", "POINTING", "Observation mode (pointing or slew)")
          call setAttribute(outDataSetH, "OBS_ID", trim(obsId), "Observation identifier")
          call setAttribute(outDataSetH, "EXP_ID", trim(expId), "Exposure identifier")

          ! Write CONTENT keyword to outSet.

          !   if (trim(dataType) .eq. "IMG.IM") then
          !      call setAttribute(outDataSetH, "CONTENT", "OM OSW IMAGE", "Content of file")
          !   else if (trim(dataType) .eq. "FST.EL") then
          !      call setAttribute(outDataSetH, "CONTENT", "OM FAST MODE EVENT LIST", "Content of file")
          !   else if (trim(dataType) .eq. "TRH.EL") then
          !      call setAttribute(outDataSetH, "CONTENT", "OM TRACKING STAR TIME SERIES", "Content of file")
          !   else
          !      call warning("modeUnknown", "OM image mode file not in image mode")
          !      call setAttribute(outDataSetH, "CONTENT", "UNKNOWN", "Content of file")
          !   endif

          call setAttribute(outDataSetH, "ORIGIN", "UNKNOWN", "Origin of FITS file")
          call setAttribute(outDataSetH, "OBJECT", trim(object), "Name of observed object")

          ! ****** JFR Need to find out if this can be taken from the observation summary file as telescope pointing etc. ******
          ! ****** Hardwired at the moment.                                                                               ******
          !MUST
          ! Will eventually use the following OAL call

          !
          ! AMK - 20/07/2000
          !
          if(.not. slewFlag) then 
             ! Get attitude information
             call OAL_setState(OM)

   
             if(OAL_getAttitude(toTimeTag(dateObs),attitude)) then
                attrarad = attitude%ra
                attdecrad = attitude%dec
                attPArad = attitude%apos

                call OAL_toAttitudeMatrix(attitude,matrix)
             endif
          end if

          call setAttribute(outDataSetH, "RA_PNT", attrarad*radian, "degrees", "RA of instrument pointing")
          call setAttribute(outDataSetH, "DEC_PNT",attdecrad*radian, "degrees", "DEC of instrument pointing")
          call setAttribute(outDataSetH, "PA_PNT", attPArad*radian, "degrees", "ROLL angle of instrument pointing")

          !//////////////////////////////////////////////////////////////////////////////////////////////////////////////

          ! Add rest of keywords to outSet.

          call setAttribute(outDataSetH, "RADECSYS", "FK5", " ")
          call setAttribute(outDataSetH, "EQUINOX", 2000, "", "Equinox of celestial coordinate system")
          call setAttribute(outDataSetH, "EXPOSURE", real(exposure), "secs","Exposure")
          call setAttribute(outDataSetH, "DATE-OBS", trim(dateObs), "Date observations were made")
          call setAttribute(outDataSetH, "DATE-END", trim(dateEnd), "Date observations ended")
          call setAttribute(outDataSetH, "OBSERVER", observer, "Name of PI")

          if (fileType == "IMI" .or. fileType == "FAE" ) then
             call  AddTimingAttributes(outDataSetH)
          end if 

      

          ! initialise binning    

          binx = 2**binax1
          biny = 2**binax2

          if (trim(dataType) .eq. "FST.EL") then
             binx=1
             biny=1
          endif

          if (binBPE) then
             binx=binx*2.0
             biny=biny*2.0
          end if

          !--------------------------------------------------------------------------------
          if ((trim(dataType) .eq. "IMG.IM") .or. (trim(dataType) .eq. "FST.EL")) then
             write(oswId, '(i10)') sciwin
             write (detSec, '(a1,i4,a1,i4,a1,i4,a1,i4,a1)') "[",windowX0/binx, ":", (windowX0 + windowDX)/binx, &
                  & ",",windowY0/biny, ":", (windowY0 + windowDY)/biny, "]" 
             write (detsize,'(a3,i4,a3,i4,a1)') "[1:", INT(2048.0/real(binx)),",1:",INT(2048.0/real(biny)), "]"

             call setAttribute(outDataSetH, "DETSIZE", trim(detsize), "Mosaic Keywords")
             call setAttribute(outDataSetH, "DETSEC", trim(detSec), "Mosaic Keywords")
          end if


          if ((trim(dataType) .eq. "IMG.IM") .or. (trim(dataType) .eq. "FST.EL")) then
             call setAttribute(outDataSetH, "OSW_ID", trim(oswId), "Identifier for current OSW")
          end if

          !
          !------------------------------------------------------------------------------------
          !
          ! AMK - 23/08/2000
          !
          ! Add the BLANK keyword
          !
          if (trim(dataType) .eq. "IMG.IM") then
             call setAttribute(outDataSetH, "BLANK", blank, "Value indicating missing pixel")
          end if
          !
          !------------------------------------------------------------------------------------
          !
          call setAttribute(outDataSetH, "BINBPE", binBPE, "BPE binning enabled")

          if (trim(dataType) .eq. "IMG.IM") then
             call setAttribute(outDataSetH, "BINAX1", binAx1, "", "CCD x-axis binning 2**N")
             call setAttribute(outDataSetH, "BINAX2", binAx2, "", "CCD y-axis binning 2**M")
          elseif(trim(dataType) .eq. "FST.EL") then
             call setAttribute(outDataSetH, "BINAX1", 0, "", "CCD x-axis binning 2**N")
             call setAttribute(outDataSetH, "BINAX2", 0, "", "CCD y-axis binning 2**M")      
             call setAttribute(outDataSetH, "BFASTID", binAx2, "", "Blue dsp 1 or 2")  
             call setAttribute(outDataSetH, "SAMPTIME", binAx1, "1/1024 secs", "Fast Mode slice time") 
          end if

          ! Not sure if these are required for Data Product.
          call setAttribute(outDataSetH, "FRMTIME", frmTime, "1/1024 secs", "Tracking frame time")
          call setAttribute(outDataSetH, "TMEXP_ID", tmExpId, "", "Exposure identifier")
          call setAttribute(OutDataSetH, "HDUCLASS", "OGIP", "Format conforms to OGIP/GSFC conventions")

          ! Add OGIP HDUCLASS Values.

          !   if (trim(dataType) .eq. "IMG.IM") then
          !      call setAttribute(OutDataSetH, "HDUCLAS1", "IMAGE", "File contains an image")
          !      call setAttribute(OutDataSetH, "HDUCLAS2", "TOTAL", "Total counts")
          !   else if (trim(dataType) .eq. "FST.EL") then
          !      call setAttribute(OutDataSetH, "HDUCLAS1", "EVENTS", "File contains an eventlist")
          !      call setAttribute(OutDataSetH, "HDUCLAS2", "ACCEPTED", "Accepted events only")
          !   else if (trim(dataType) .eq. "TRH.EL") then
          !      call setAttribute(OutDataSetH, "HDUCLAS1", "LIGHTCURVE", "File contains an eventlist")
          !      call setAttribute(OutDataSetH, "HDUCLAS2", "TOTAL", "Accepted events only")
          !   else
          !      call setAttribute(OutDataSetH, "HDUCLAS1", "UNKNOWN", "")
          !      call setAttribute(OutDataSetH, "HDUCLAS2", "UNKNOWN", "")
          !   endif

          ! Write data to outSet.

          !***********************************************************
          ! Start of IMG.IM if block.
          !*********************************************************** 
          IMGIM: if (trim(dataType) .eq. "IMG.IM") then
             ! Convert from int32 data in odf to real32 data for output.
             outArrayH = addArray(outDataSetH, "PRIMARY", real32, imageDim)

             real32ImageP => real32Array2Data(outArrayH)
             real32ImageP = real(int32ImageP)

             outArrayH = addArray(outDataSetH, "RAW", real32, imageDim)

             real32ImageP => real32Array2Data(outArrayH)
             real32ImageP = real(int32ImageP)

          else if (trim(dataType) .eq. "FST.EL") then
             outTableH = addTable(outDataSetH, "OMFAE1", nEvents)

             call copyAllAttributes(attributable(inTableH), attributable(outTableH))

             outColumnH(1) = addColumn(outTableH, "FRAME", integer32)
             int32ColumnP => int32Data(outColumnH(1))
             int32ColumnP = frameP
             outColumnH(2) = addColumn(outTableH, "FTCOARSE", integer32)
             int32ColumnP => int32Data(outColumnH(2))
             int32ColumnP = ftCoarseP
             outColumnH(3) = addColumn(outTableH, "FTFINE", integer16)
             int16ColumnP => int16Data(outColumnH(3))
             int16ColumnP = ftFineP
             outColumnH(4) = addColumn(outTableH, "RAWX", integer16)
             int16ColumnP => int16Data(outColumnH(4))
             int16ColumnP = rawXP

             call setAttribute(outColumnH(4), "TLMIN", 0)
             call setAttribute(outColumnH(4), "TLMAX", windowDX - 1)

             outColumnH(5) = addColumn(outTableH, "RAWY", integer16)
             int16ColumnP => int16Data(outColumnH(5))
             int16ColumnP = rawYP

             call setAttribute(outColumnH(5), "TLMIN", 0)
             call setAttribute(outColumnH(5), "TLMAX", windowDy - 1)

          else if (trim(dataType) .eq. "TRH.EL") then

             outTableH = addTable(outDataSetH, "OMTHX1", nFrames)

             call copyAllAttributes(attributable(inTableH), attributable(outTableH))

             !
             ! AMK - 15/07/2000
             !
             ! Reset the start and end times to the correct values
             !
             call setAttribute(outTableH, "DATE-OBS", trim(dateObs), "Start time of exposure")
             call setAttribute(outTableH, "DATE-END", trim(dateEnd), "End time of exposure")

             columnDim(1) = 1
             outColumnH(1) = addColumn(outTableH, "FRAME", integer32, "COUNTER", columnDim)
             int32ColumnP => int32Data(outColumnH(1))
             int32ColumnP = frameP
             outColumnH(2) = addColumn(outTableH, "NGGS", integer32, "COUNTS", columnDim)
             int32ColumnP => int32Data(outColumnH(2))
             int32ColumnP = nggsP
             outColumnH(3) = addColumn(outTableH, "DX", integer32, " ", columnDim)
             int32ColumnP => int32Data(outColumnH(3))
             int32ColumnP = dXP
             outColumnH(4) = addColumn(outTableH, "DY", integer32, " ", columnDim)
             int32ColumnP => int32Data(outColumnH(4))
             int32ColumnP = dYP
             outColumnH(5) = addColumn(outTableH, "ROLL", integer32, " ", columnDim)
             int32ColumnP => int32Data(outColumnH(5))
             int32ColumnP = rollP
             outColumnH(6) = addcolumn(outTableH, "QUALITY", integer32, " ", columnDim)
             int32ColumnP => int32Data(outColumnH(6))
             int32ColumnP = qualityP

             !////////////////////////////////////////////////////////////////////////////////
             !// AMK - PATCH 17-05-2000
             !// Altered the value of columnDim(1) which is the size of the guide star array
             !// as there was some confusion here. We have seen data sets that use 10 and some
             !// that use 16. This fix works out the size use and sets the array size 
             !// appropriately. This should now work in all cases.
             !///////////////////////////////////////////////////////////////////////////////
             !      columnDim(1) = 10
             columnDim(1) = size(gsXP)/size(nggsP)
             outColumnH(7) = addColumn(outTableH, "GSX", integer32, "PIXEL", columnDim)
             int32VectColumnP => int32VectorData(outColumnH(7))
             int32VectColumnP = gsXP
             outColumnH(8) = addColumn(outTableH, "GSY", integer32, "PIXEL", columnDim)
             int32VectColumnP => int32VectorData(outColumnH(8))
             int32VectColumnP = gsYP
             outColumnH(9) = addColumn(outTableH, "GSCTS", integer32, "COUNTS", columnDim)
             int32VectColumnP => int32VectorData(outColumnH(9))
             int32VectColumnP = gsCtsP
             !********************************************
             ! End of IMG.IM if block.
             !*******************************************
          end if IMGIM

          !************************************
          ! Start of IMG_FST if block
          !************************************

          IMG_FST: if ((trim(dataType) .eq. "IMG.IM") .or. (trim(dataType) .eq. "FST.EL")) then

             ! Add a MODES extension, detailing the science window parameters.

             modesTableH = addTable(outDataSetH, "MODES", 1)

             columnDim(1) = 1

             if (trim(dataType) .eq. "IMG.IM") then
                mode = 0
             else if (trim(dataType) .eq. "FST.EL") then
                mode = 1
             endif

             modesColumnH(1) = addColumn(modesTableH, "MODE", Integer32, "none", columnDim)
             modeP => int32Data(modesColumnH(1))
             modeP = mode

             modesColumnH(2) = addColumn(modesTableH, "WINDOWX0", Integer32, "pixels", columnDim)
             windowX0P => int32Data(modesColumnH(2))
             windowX0P = windowX0

             modesColumnH(3) = addColumn(modesTableH, "WINDOWY0", Integer32, "pixels", columnDim)
             windowY0P => int32Data(modesColumnH(3))
             windowY0P = windowY0

             modesColumnH(4) = addColumn(modesTableH, "WINDOWDX", Integer32, "pixels", columnDim)
             windowDXP => int32Data(modesColumnH(4))
             windowDXP = windowDX

             modesColumnH(5) = addColumn(modesTableH, "WINDOWDY", Integer32, "pixels", columnDim)
             windowDYP => int32Data(modesColumnH(5))
             windowDYP = windowDY

             ! Release handles on data.
             call release(modesColumnH(1))
             call release(modesColumnH(2))
             call release(modesColumnH(3))
             call release(modesColumnH(4))
             call release(modesColumnH(5))
             call release(modesTableH)
             !*****************************************************
             ! End of IGF_FST if block
             !*****************************************************
          end if IMG_FST

          ! 24th March 2000 JFR
          !*********************************************************
          ! End of ZERODRIFT if block.
          !*********************************************************
       end if  ZERODRIFT
       !
       ! AMK - 21/07/2000
       !
       ! Added
       !
    else
       outDataSetH = dataSet(outSet, create)

       outTableH = addTable(outDataSetH, "ZERODRIFT", 1)

       !      call copyAllAttributes(attributable(inTableH), attributable(outTableH))

       outColumnH(1) = addColumn(outTableH, "DRIFT", integer32)
       int32ColumnP => int32Data(outColumnH(1))
       int32ColumnP = 0.0

       call release(outColumnH(1))
       call release(outTableH)
    end if THXFLAG1

    if (fileType == "IMI") then
       call release(outArrayH)
       call release(inArrayH)

    else if (fileType == "FAE") then
       call release(inColumnH(1))
       call release(inColumnH(2))
       call release(inColumnH(3))
       call release(inColumnH(4))
       call release(inColumnH(5))
       call release(inTableH)
       call release(outcolumnH(1))
       call release(outcolumnH(2))
       call release(outcolumnH(3))
       call release(outcolumnH(4))
       call release(outcolumnH(5))
       call release(outTableH)

    else if (fileType == "THX") then
       call release(inColumnH(1))
       call release(inColumnH(2))
       call release(inColumnH(3))
       call release(inColumnH(4))
       call release(inColumnH(5))
       call release(inColumnH(6))
       call release(inColumnH(7))
       call release(inColumnH(8))
       call release(inColumnH(9))
       call release(inTableH)
       call release(outcolumnH(1))
       call release(outcolumnH(2))
       call release(outcolumnH(3))
       call release(outcolumnH(4))
       call release(outcolumnH(5))
       call release(outcolumnH(6))
       call release(outcolumnH(7))
       call release(outcolumnH(8))
       call release(outcolumnH(9))
       call release(outTableH)
    end if

    !
    ! AMK - 21/07/2000
    !
    ! Added
    !
    if(.not. dummyTHXFlag) then
       call release(inDataSetH)
    end if

    call release(outDataSetH)

    !**************************************************************
    ! AMK - 21/07/2000
    !
    ! Added. Need to create a dummy tracking file if in slew mode
    !**************************************************************
    if(slewFlag) then
       outDataSetH = dataSet("tmp_tracking", create) 

       outTableH = addTable(outDataSetH, "ZERODRIFT", 1)

       !      call copyAllAttributes(attributable(inTableH), attributable(outTableH))

       outColumnH(1) = addColumn(outTableH, "DRIFT", integer32)
       int32ColumnP => int32Data(outColumnH(1))
       int32ColumnP = 0.0

       call release(outColumnH(1))
       call release(outTableH)

       call release(outDataSetH)
    end if

  end subroutine omprep
!**************************************************
! subroutine getExposureStartRow
!
!**************************************************
subroutine getExposureStartRow(pehTableH, numberOfRows, startTime, startRow)

    type(TableT),intent(in) :: pehTableH
    type(ColumnT)           :: rowTime
    character (len=23),intent(in) :: startTime
    integer, intent(in) :: numberOfRows
    integer, intent(out)       :: startRow
    integer i
 
!    write(*, *) "Sub getExposureStartRow - startTime = ", startTime, " ", ToTimeTag(startTime)

    !****************************************************************************
    ! Loop over all the rows until we find the row that gives us the correct time
    !
    !****************************************************************************
    rowTime = column(pehTableH, "TIME" ,read)
    
    do i = 0, numberOfRows - 1
       if(OAL_ertToTimeTag(stringCell(rowTime, i)) >= ToTimeTag(startTime)) then
 
          startRow = i
          call release(rowTime)
          return
       end if

    end do
    !********************************
    ! Won't reach here if time found
    !********************************

    call release(rowTime)
    call fatal("starttime greater than times in peh file")
    

  end subroutine getExposureStartRow
!*********************************************
! ConvertToUpperCase - Converts any lower-case
! characters in a string to upper-case characters.
!*********************************************
subroutine ConvertToUpperCase(string)
    character(len=*), intent(inout) :: string
    character(len=1) :: cha
    integer i, newchar

    do i = 1, len(string)
        cha = string(i : i)
        if (lge(cha, 'a') .and. lle(cha, 'z')) then
            newchar = ichar(cha) - ichar('a') + ichar('A')
    
            string(i : i) = char(newchar)
        end if
    end do
end subroutine ConvertToUpperCase
!************************************************
! IsBadFilter - Checks to see if  a given filter is
! an invalid one- badFilter set to  TRUE if so- FALSE
! otherwise
!************************************************
logical function IsBadFilter(filterString)
    character(len=*), intent(inout) :: filterString
    

    select case(trim(filterString))

        case ("BLOCKED")
            IsBadFilter = .true.

        case("UNKNOWN")
            IsBadFilter = .true.

        case("GRISM2")
            IsBadFilter = .true.

        case("MAGNIFIER")
            IsBadFilter = .true.

        case("GRISM1")
            IsBadFilter = .true.

        case default
            IsBadFilter = .false.

    end select

end function IsBadFilter
!***************************************************************
! subroutine AddTimingAttributes
!**************************************************************
subroutine AddTimingAttributes(dataSetH)
   type(DataSetT), intent(in) :: dataSetH

   call setAttribute(dataSetH, "TIMESYS", "TT", "XMM time will be TT (Terrestial time)")
   call setAttribute(dataSetH, "MJDREF", 50814.0, "d", "1998-01-01T00:00:00 (TT) expressed in MJD (TT)")
   call setAttribute(dataSetH, "TIMEZERO", 0, "s", "Clock correction (if not zero)")
   call setAttribute(dataSetH, "TIMEUNIT", "s", "All times in s unless specified otherwise")
   call setAttribute(dataSetH, "CLOCKAPP", "T", "Clock correction applied")
   call setAttribute(dataSetH, "TIMEREF", "LOCAL", "Reference location of photon arrival times")
   call setAttribute(dataSetH, "TASSIGN", "SATELLITE", "Location of time assignment")

end subroutine AddTimingAttributes


!*********************************************
! Subroutine  CheckImageSizeOkay
! Checks to see if the image size corresponds
! to what the program expects it to be. 
!*********************************************
subroutine CheckImageSizeOkay(binBPE, windowDx, windowDy, &
                  binXaxis, binYaxis, xAxisLength, yAxisLength)
    logical, intent(in) :: binBPE
    integer(kind=int32), intent(in) :: windowDx
    integer(kind=int32), intent(in) :: windowDy
    integer(kind=int32), intent(in) :: binXaxis
    integer(kind=int32), intent(in) :: binYaxis
    integer(kind=int32), intent(in) :: xAxisLength
    integer(kind=int32), intent(in) :: yAxisLength
    integer(kind=int32) :: factor

    if (binBPE) then
        factor = 2
    else
        factor = 1
    end if

    if ((windowDx / (binXaxis * factor)) .ne. xAxisLength) then
        call fatal("X Image size computed from BINPE, WINDOWDX and BINAX1 does not equal NAXIS1")
    end if


    if ((windowDy / (binYaxis * factor)) .ne. yAxisLength) then
        call fatal("Y Image size computed from BINPE, WINDOWDY and BINAX2 does not equal NAXIS2")
        
        
    end if

end subroutine CheckImageSizeOkay
 

end module omprep_mod









